from lxml import etree

import glob
import os
import time
import sys

import docing
import filing

from emailer import Emailer
from nitpo import Nitpo
from infile import Infile
from itresu import Itresu
from omnibus import Omnibus
from kapro import Kapro
from reports import Reports
from xpaths import Xpaths
from profile import Profile
from surks import Surks

class Repis(Nitpo):

    def __init__(self, do_verbose=False):
        """report issues"""
        super().__init__()
        # a basename for the thing to send
        self.base = None
        # current repcode
        self.repcode = None
        # # mtime of the fufi, only use to skip for very
        # # recent surks
        self.fufi_mtime = None
        # state variable for the current fufi
        self.repis_fufi = None
        self.omnibus = Omnibus(do_verbose=do_verbose)
        self.emailer = Emailer(do_verbose=do_verbose)
        self.infile = Infile(do_verbose=do_verbose)
        self.itresu = Itresu(do_verbose=do_verbose)
        self.xpaths = Xpaths(do_verbose=do_verbose)
        self.kapro = Kapro(do_verbose=do_verbose)
        self.profile = Profile()
        self.surks = Surks()
        # # orimas for additional deduping, read only for
        # # sending to profiles
        self.orimas = None
        if not self.has_conf('folders', 'mail'):
            raise Exception("I need a Ф[folders][mail].")
        self.do_verbose = do_verbose

    def run(self, fufi, only_emad=None):
        self.repis_fufi = fufi
        self.base = self.infile.get_base(fufi)
        self.repcode = self.infile.get_repcode(fufi)
        if not self.is_dev():
            self.kapro.update(self.repcode)
        self.send_omnibus(fufi)
        self.fufi_mtime = filing.mtime(fufi)
        if not self.has_conf('repis', 'to_profiles'):
            return
        if not self.conf['repis']['to_profiles'] == 'true':
            return
        reports = Reports()
        orimas = reports.get_orimas(get_cache_only=True)
        self.orimas = orimas['e']
        self.send_to_profiles(fufi, only_emad=only_emad)

    def send_to_profiles(self, fufi, only_emad=None):
        # should be done by the calling bin
        # self.kapro.update(repcode)
        emads = self.kapro.load(self.repcode)
        # print(emads)
        for emad in emads:
            # print(emad)
            if only_emad is not None and emad != only_emad:
                continue
            self.send_to_profile(fufi, emad)

    def send_to_profile(self, fufi, emad):
        surks_docu = self.profile.load(emad)
        # # get the repcodes in the profile
        repcodes = self.profile.repcodes(surks_docu)
        # # don't send if the time on the surk is earlier
        # # than the time on the repis fufi. It's a simple
        # # way to avoid old issues going to new subscribers
        if self.repcode in repcodes:
            repcode = self.repcode
            from_xp = '/n:profile/n:live/n:surk'
            from_xp += f'[@repcode="{repcode}"]/@from'
            the_from = self.xpaths.run(surks_docu, from_xp)
            if not the_from:
                print("repis sees no surk @from", file=sys.stderr)
                return
            if the_from[0] > self.fufi_mtime:
                if sys.stdin and sys.stdin.isatty():
                    print(f"repis skips {repcode} because surk is too new.")
                    return False
        # # take account of the orima
        count_sleep = 0
        if self.has_conf('repis', 'wait'):
            count_sleep = int(self.conf['repis']['wait'])
        if emad in self.orimas:
            for repcode in self.orimas[emad]:
                repcodes[repcode] = 1
        # # now call the itresu, fills itresu.dups
        self.itresu.get_dups(fufi)
        if self.itresu.are_any_left(fufi, repcodes):
            out = self.send(emad, surks_docu.getroot(), only=repcodes)
            if out is not None and count_sleep > 0:
                time.sleep(count_sleep)
        else:
            print(f"repis: no items left to send to {emad}.")
        return True

    def send_omnibus(self, fufi):
        omnibus_doc = self.omnibus.read()
        if omnibus_doc is None:
            # print("repis: I have no omnibus.")
            return
        surk_docus = self.omnibus.disect(omnibus_doc)
        if len(surk_docus) == 0:
            return None
        # # avoid running prepare every time we send a mail
        mail_base_fufi = self.conf['folders']['mail'] + '/' + self.base
        # # now call the itresu, fills itesu.dups
        self.itresu.get_dups(fufi)
        if not self.itresu.are_any_left(fufi, None):
            print("repis: no items left to send for omnibus.")
            return None
        filing.prepare(mail_base_fufi)
        # get the subs for the itresu
        for emad in surk_docus:
            self.send(emad, surk_docus[emad], is_omnibus=True)
        # docing.inject(fufi, omnibus_doc)

    def send(self, emad, surk_docu, only=None, is_omnibus=False):
        base = self.base + '/' + self.profile.relfi_from_emad(emad)[:-4]
        maix = filing.parse_lax(self.repis_fufi)
        if maix is None:
            print(f"repis can not parse {self.repis_fufi}",
                  file=sys.stderr)
            return None
        surk_ele = self.surks.set_ele(surk_docu, self.repcode, emad)
        if surk_ele is None:
            # # happen on 2022-07-13 for first time, some error in the_infile
            # # coordination of the data
            print(f"repis does not see a surk for {emad}.")
            return False
        sint = self.surks.sint(surk_ele)
        maix_surk = docing.inject(maix, surk_docu)
        # spro = self.xpaths.get_spro(surk_docu, repcode=self.repcode)
        # # only for the omnibus, the spros=a means no dedup
        # if not (is_omnibus and spro == 'a'):
        if sint == 's':
            # # None is all repcodes here, omnibus only
            subs_ele = self.itresu.xml(self.repis_fufi, only)
            maix_surk = docing.inject(maix, subs_ele)
        # ## the mail xml
        out = self.emailer.prepare(maix_surk, base=base, only=only)
        return out
        #print(spro)
